import warnings
warnings.filterwarnings("ignore")

import numpy as np

import matplotlib.pyplot as plt
import seaborn as sns

from ..style.formats import _thousand_sep
from ..style.titles import _titles
from ..style.template import _header, _footer


def corr(data,
         corr_method='spearman',
         annot=False,
         mask=True,
         line_width=1,
         line_color='black',
         color_grades=5,
         auto_sizing=True,
         palette='default',
         style='astetik',
         dpi=72,
         title='',
         sub_title='',
         x_label='',
         y_label='',
         legend=True,
         x_scale='linear',
         y_scale='linear',
         x_limit=None,
         y_limit=None,
         save=False):

    '''CORRELATION HEATMAP

    This is best used with less than 50 variables in the dataset.
    For best results, column labels should be clear and not too long.

   Inputs: a dataframe with several columns
   Features: Both categorical and continuous features will be used

    1. USE
    ======
    ast.compare(data=patients,
                x='hospital_stays',
                y=['died_hospital','died_out'],
                label_col='religion',
                transform=True)

    1. USE
    ======
    ast.box(data=patients,
            x='insurance',
            y='age',
            hue='expired')

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: a pandas dataframe

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
    corr_method :: The method that will be used for the correlation:
                    - 'pearson' : standard correlation coefficient
                    - 'kendall' : Kendall Tau correlation coefficient
                    - 'spearman' : Spearman rank correlation

    annotation :: True if each cell will be annotated with the value

    mask :: If set to False, a rectangular shape will be drawn instead
            of a triangular shapeself.

    line_width :: the width of the white lines between each element. Better
                  to set small when there are really many items.

    line_color :: the color of the lines between the elements e.g. 'black'

    auto_sizing :: If not True, then should be int value in inches which is used
                   for both width and height.

    color_grades :: The number of colors/shades to use in total. 5 is default.
                    Generally the best results come with 3 or 5 or 7, but looks
                    better with more colors.

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the astetik palettes:
                'default'
                'colorblind'
                'blue_to_red'
                'blue_to_green'
                'red_to_green'
                'green_to_red'
                'violet_to_blue'
                'brown_to_green'
                'green_to_marine'

                Or use any cmap, seaborn or matplotlib
                color or palette code, or hex value.

    style :: Use one of the three core styles:
                'astetik'     # white
                '538'         # grey
                'solarized'   # sepia

              Or alternatively use any matplotlib or seaborn
              style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'
    '''

    # # # # # PREP STARTS # # # # #
    data = data.corr(method=corr_method)

    if mask == True:
        mask = np.zeros_like(data)
        mask[np.triu_indices_from(mask)] = True
        line_color = 'white'
    else:
        mask = None
    # # # # # PREP ENDS # # # # #

    # HEADER STARTS >>>
    palette = _header(palette,
                      style,
                      n_colors=color_grades,
                      dpi=dpi)

    if auto_sizing == True:
        size = data.shape[0] / 2 + 5

    # PLOT
    p, ax = plt.subplots(figsize=(size, size))

    p = sns.heatmap(data,
                    mask=mask,
                    linewidths=line_width,
                    linecolor=line_color,
                    cmap=palette,
                    annot=annot)

    # HEADER
    _titles(title, sub_title=sub_title)
    _footer(p, x_label, y_label, save=save, tight=False, despine=False)

    p.set_xticklabels(data, rotation=90)
    p.set_yticklabels(data, rotation=0)



import time
from datetime import datetime

import seaborn as sns
import matplotlib.pyplot as plt
from matplotlib.pyplot import rcParams
from ..style.color_picker import color_picker, color_blind, _label_to_hex
from ..utils.utils import _n_decider
from ..style.style import styles, default_colors
from ..style.random_colors import randomcolor


def _header(palette,
            style,
            n_colors,
            dpi,
            fig_width=None,
            fig_height=None):

    if palette == 'random':
        palette = randomcolor()

    elif style != 'astetik':
        plt.style.use(style)

    n = _n_decider(n_colors)

    try:
        if palette.startswith('colorblind'):
            palette = color_blind(palette)
    except AttributeError:
        palette = palette

    else:
        try:
            palette = color_picker(palette=palette, n_colors=n)
        except UnboundLocalError:
            palette = _label_to_hex(palette, n_colors=n)

    if fig_height != None and fig_width != None:
        plt.figure(figsize=(fig_width, fig_height))

    if style == 'astetik':
        rcParams['figure.facecolor'] = 'white'
        rcParams['axes.facecolor'] = 'white'

    style_dic = styles(dpi)
    for key in style_dic.keys():
        rcParams[key] = style_dic[key]

    return palette


def _footer(p,
            xlabel,
            ylabel,
            legend=False,
            n=None,
            save=False,
            tight=True,
            despine=True):

    default_color = default_colors()

    # LEGEND STARTS >>>
    if legend != False:
        plt.legend(loc='upper right', bbox_to_anchor=(1.3, 0.9), ncol=1)

    # HANDLE LABELS
    plt.xlabel(xlabel, color=default_color)
    plt.ylabel(ylabel, color=default_color)

    # DESPINE THE FIGURE
    if despine == True:
        try:
            p.spines['bottom'].set_color('black')
        except:
            pass
        sns.despine(left=True, right=True, top=True)

    # FORCE TIGHT LAYOUT
    if tight == True:
        plt.tight_layout()

    # SAVING THE PLOT
    if save != False:

        if isinstance(save, str):
            filename = save
        
        else:
            dt = datetime.now()
            time_stamp = time.strftime('%Y%m%d_%H%M%S_' + str(dt.microsecond))
            filename = "astetik_" + time_stamp + ".png"
        
        plt.savefig(filename, dpi=72)



import matplotlib.pyplot as plt


def _titles(title,
            sub_title,
            location='center',
            fontname='Arial',
            fontsize='18'):

    '''TITLE HANDLER

    Takes care of the extremely painful task of positioning
    various titles and labels dynamically regardless of the plot.
    Or at least that's the idea...

    USE
    ===
    _title_handling(p, data, title, sub_title, samplenote, footnote)

    PARAMETERS
    ==========
    p :: the figure object
    data :: the data that is used in the plot
    title :: title string object or None
    sub_title :: sub_title string object or None
    footnote :: string object or None
    samplenote :: string object or None

    NOTE: At the moment works with one dimensional data.

    '''
    if len(title) + len(sub_title) > 0:
        title = title.replace(' ', '\,')
        title = title.replace('_', '\_')

        plt.title(r"$\bf{" + title + "}$" + '\n' + sub_title,
                  loc=location,
                  fontsize=fontsize,
                  fontname=fontname,
                  weight='normal',
                  y=1.03,
                  color="#342b3b");


                  




import seaborn as sns

from ..style.template import _header, _footer
from ..utils.utils import _scaler
from ..utils.utils import factorplot_sizing
from ..style.titles import _titles


def bartwo(data,
           x,
           y,
           hue=None,
           sort=None,
           multi_color=False,
           error_bars=None,
           palette='default',
           style='astetik',
           dpi=72,
           title='',
           sub_title='',
           x_label='',
           y_label='',
           legend=True,
           x_scale='linear',
           y_scale='linear',
           x_limit=None,
           y_limit=None,
           save=False):

    '''2-D BAR PLOT

    A 2-dimensional bar graph for the case where there is a continuous
    variable that is compared against labels and a single categorial. The
    'x' is the continuous variable, 'y' is the labels (categorical) and
    'hue' is the comparison color.

    1. USE
    ======
    ast.bar1d(data=patients,
              x='icu_days',
              y='insurance')

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: pandas dataframe

    x :: x-axis data (continuous)

    y :: y-axis data (category labels)

    hue :: color comparison data (preferably binary categorical)

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
    sort :: If True, values will be sorted ascending, if False descending.

    multi_color :: If True, label values will be used for hue.

    error_bars :: 'sd' for using standard deviation and a float value for
                  using bootstrapping.

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the astetik palettes:
                'default'
                'colorblind'
                'blue_to_red'
                'blue_to_green'
                'red_to_green'
                'green_to_red'
                'violet_to_blue'
                'brown_to_green'
                'green_to_marine'

                Or use any cmap, seaborn or matplotlib
                color or palette code, or hex value.

    style :: Use one of the three core styles:
                'astetik'     # white
                '538'         # grey
                'solarized'   # sepia

              Or alternatively use any matplotlib or seaborn
              style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'

    '''
    size, aspect = factorplot_sizing(data[y], width=7, thickness=2, auto=True)
    n_colors = len(data[hue].unique())

    # HEADER STARTS >>>
    palette = _header(palette,
                      style,
                      n_colors=n_colors,
                      dpi=dpi,
                      fig_height=None,
                      fig_width=None)
    # <<< HEADER ENDS
    p = sns.factorplot(data=data,
                       x=x,
                       y=y,
                       hue=hue,
                       palette=palette,
                       aspect=aspect,
                       size=size,
                       kind='bar',
                       orient='h',
                       ci=error_bars)

    # SCALING AND LIMITS STARTS >>>
    if x_scale != 'linear' or y_scale != 'linear':
        _scaler(p, x_scale, y_scale)

    # FOOTER STARTS >>>
    _titles(title, sub_title=sub_title)
    _footer(p, x_label, y_label, save=save)

    if data[x].min() < 0:
        sns.despine(left=True)



import matplotlib.pyplot as plt

from ..utils.exceptions import MissingLabel
from ..style.titles import _titles
from ..style.template import _header, _footer


def pie(data,
        x=None,
        labels=None,
        palette='default',
        style='astetik',
        dpi=72,
        title='',
        sub_title='',
        x_label='',
        y_label='',
        legend=True,
        x_scale='linear',
        y_scale='linear',
        x_limit=None,
        y_limit=None,
        save=False):

    '''PIE PLOT

    A classic pie chart. If dataframe is given as input, column sums
    will be automatically used as values. If you want to use a given
    row of data instead, indicate it using data.iloc[4] in the case of
    wanting to use the 5th row. If array is given as input, provide labels
    separately.

    Inputs: 1
    Features: 1 categorical or continuous

    1. USE
    ======
    ast.pie(data=patients, x='age', palette='Blues')

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: pandas dataframe

    x :: x-axis data (categorical or continuous)

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
    NONE

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the hand-crafted palettes:
                'default'
                'colorblind'
                'blue_to_red'
                'blue_to_green'
                'red_to_green'
                'green_to_red'
                'violet_to_blue'
                'brown_to_green'
                'green_to_marine'

                Or use any cmap, seaborn or matplotlib
                color or palette code, or hex value.

    style :: Use one of the three core styles:
                'astetik'     # white
                '538'         # grey
                'solarized'   # sepia

              Or alternatively use any matplotlib or seaborn
              style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'

    '''

    if x != None:

        # more than single column
        try:
            data[x].shape[1]
            labels = data[x].sum().index.values
            data = data[x].sum().values

        # single column
        except IndexError:
            labels = data[x].value_counts().index.values
            data = data[x].value_counts().values

    else:
        try:
            data.shape[1]
            labels = data.sum().index.values
            data = data.sum().values
        except IndexError or ValueError:
            pass

    n_colors = len(data)

    # HEADER STARTS >>>
    palette = _header(palette, style, n_colors, dpi)
    # <<< HEADER ENDS

    p, ax = plt.subplots(figsize=(8, 8))

    # # # # # # PLOT CODE STARTS # # # # # #
    p = plt.pie(x=data,
                colors=palette,
                autopct='%.1f%%',
                pctdistance=1.3,
                startangle=90)
    plt.axis('equal')
    # # # # # # PLOT CODE ENDS # # # # # #

    # LEGEND STARTS >>>

    try:
        if legend != False:
            plt.legend(p[0], labels, loc='center left', bbox_to_anchor=(1.1, 0.5))
    except TypeError:
        MissingLabel("Looks like you didn't provide 'label' parameter for legend")

    # <<< LEGEND ENDS

    # START OF TITLES >>>
    _titles(title, sub_title=sub_title)
    _footer(p, x_label, y_label, save=save)



def twod(x, y=None, plot_title=''):

    '''2-d visualization for 1-d or 2-d random sequences.
    In the case of input being 1-d sequence, it will be split
    in half to fit into 2-d plane.
    '''

    import numpy as np
    import matplotlib.pyplot as plt

    if y is None:

        # split two columns in x into x and y
        if len(np.array(x).shape) == 2:
            y = x[:, 1]
            x = x[:, 0]

        # split 1d x into two columns
        else:
            x_len = int(len(x) / 2)
            y = x[x_len:]
            x = x[:x_len]

    plt.scatter(x=x, y=y, color='black', s=8, alpha=.4)
    plt.title(plot_title)



import seaborn as sns

import matplotlib.pyplot as plt

from ..style.style import params
from ..style.titles import _titles
from ..style.formats import _thousand_sep
from ..style.template import _header, _footer
from ..utils.utils import _limiter, _scaler


def strip(data,
          x,
          y,
          hue,
          jitter=1,
          dodge=False,
          palette='default',
          style='astetik',
          dpi=72,
          title='',
          sub_title='',
          x_label='',
          y_label='',
          legend=True,
          x_scale='linear',
          y_scale='linear',
          x_limit=None,
          y_limit=None,
          save=False):

    '''STRIP PLOT

    Similar to swarm but now there can be overlapping of observations and
    many more x categorical can be included. Also in this plot 'hue' is
    not optional.

    Inputs: 3
    Features: 1 continuous and 2 categoricals, one should be not more than
              a few unique values (hue).

    1. USE
    ======
    ast.strip(data=patients,
              x='religion',
              y='age',
              hue='expired',
              dodge='true',
              palette='colorblind')

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: pandas dataframe

    x :: x-axis data (categorical)

    y :: y-axis data (continuous or categorical)

    hue :: color highlight (categorical)

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
    jitter :: an integer value that defines the lateral movement
              of observations. This will effect the visual outputself.

    dodge :: If True then 'hue' values will be separated in their own
             stacks.

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the hand-crafted palettes:
                'default'
                'colorblind'
                'blue_to_red'
                'blue_to_green'
                'red_to_green'
                'green_to_red'
                'violet_to_blue'
                'brown_to_green'
                'green_to_marine'

                Or use any cmap, seaborn or matplotlib
                color or palette code, or hex value.

    style :: Use one of the three core styles:
                'astetik'     # white
                '538'         # grey
                'solarized'   # sepia

              Or alternatively use any matplotlib or seaborn
              style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'

    '''

    n = len(data[hue].unique())

    # HEADER STARTS >>>
    palette = _header(palette, style, n_colors=n, dpi=dpi)  # NOTE: y exception
    # <<< HEADER ENDS
    p, ax = plt.subplots(figsize=(params()['fig_width'],
                                  params()['fig_height']))
    order = data[x].unique()
    p = sns.stripplot(data=data,
                      x=x,
                      y=y,
                      hue=hue,
                      palette=palette,
                      size=4,
                      order=order,
                      dodge=dodge,
                      jitter=jitter)

    # SCALING AND LIMITS STARTS >>>
    if x_scale != 'linear' or y_scale != 'linear':
        _scaler(p, x_scale, y_scale)

    if x_limit != None or y_limit != None:
        _limiter(data=data, x=x, y=y, x_limit=None, y_limit=y_limit)

    # FOOTER STARTS >>>
    _thousand_sep(p, ax, data, x, y)
    _titles(title, sub_title=sub_title)
    _footer(p, x_label, y_label, legend, n, save)

    p.spines['bottom'].set_color('black')
    p.set_xticklabels(order, rotation=90)



def toggle():

    from IPython.display import HTML

    html = HTML('''<script>
    code_show=true;
    function code_toggle() {
     if (code_show){
     $('div.input').hide();
     } else {
     $('div.input').show();
     }
     code_show = !code_show
    }
    $( document ).ready(code_toggle);
    </script>
    <a href="javascript:code_toggle()">toggle code cells</a>''')
    return html


def warning():

    import warnings
    warnings.filterwarnings('ignore')



# PLOTS
from .plots.corr import corr
from .plots.hist import hist
from .plots.kde import kde
from .plots.pie import pie
from .plots.swarm import swarm
from .plots.scat import scat
from .plots.line import line
from .plots.grid import grid
from .plots.box import box
from .plots.violin import violin
from .plots.strip import strip
from .plots.count import count
from .plots.bargrid import bargrid
from .plots.overlap import overlap
from .plots.multikde import multikde
from .plots.compare import compare
from .plots.multicount import multicount
from .plots.bar import bar
from .plots.bartwo import bartwo
from .plots.animate import Animation
from .plots.world import world
from .plots.regs import regs
from .plots.roc import roc
# from .plots.words import words

from .utils.gui import toggle as toggle
from .utils.gui import warning as warning

# TABLES
from .tables.table import table
from .tables.text import text
# from .tables.timeseries import timeseries

try:
    del plots
except:
    pass

try:
    del style
except:
    pass

try:
    del tables
except:
    pass

try:
    del utils
except:
    pass



import seaborn as sns
import matplotlib.pyplot as plt

from ..style.formats import _thousand_sep
from ..style.style import params
from ..style.titles import _titles
from ..style.template import _header, _footer
from ..utils.utils import _limiter, _scaler


def swarm(data,
          x,
          y,
          hue=None,
          palette='default',
          style='astetik',
          dpi=72,
          title='',
          sub_title='',
          x_label='',
          y_label='',
          legend=True,
          x_scale='linear',
          y_scale='linear',
          x_limit=None,
          y_limit=None,
          save=False):

    '''SWARM PLOT
    This is similar to scatter plot, in the sense that overlapping observations
    are avoided (each sample will be seen as a dot), but observations are
    grouped based on 'x'.

    NOTE: this plot takes time to draw, so if your sample is tens or hundreds
          of thousands of observations, be patient. Or if it's not that important
          that observations do not overlap, try strip() instead.

    Inputs: 2 to 3
    Features: 1 continuous and 1 or 2 categoricals

    1. USE
    ======
    ast.violin(data=patients,
                x='insurance',
                y='age',
                hue='expired',
                y_limit=None)

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: pandas dataframe

    x :: x-axis data (categorical)

    y :: y-axis data (continuous or categorical)

    hue :: color highlight (categorical)

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
    None

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the hand-crafted palettes:
                'default'
                'colorblind'
                'blue_to_red'
                'blue_to_green'
                'red_to_green'
                'green_to_red'
                'violet_to_blue'
                'brown_to_green'
                'green_to_marine'

                Or use any cmap, seaborn or matplotlib
                color or palette code, or hex value.

    style :: Use one of the three core styles:
                'astetik'     # white
                '538'         # grey
                'solarized'   # sepia

              Or alternatively use any matplotlib or seaborn
              style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'

    '''

    # PLOT SPECIFIC START >>>
    if hue != None:
        n = len(data[hue].unique())
    else:
        legend = False
        n = 1
    # <<< PLOT SPECIFIC ENDS

    # HEADER STARTS >>>
    palette = _header(palette, style, n_colors=n, dpi=dpi)  # NOTE: y exception
    # <<< HEADER ENDS
    p, ax = plt.subplots(figsize=(params()['fig_width'],
                                  params()['fig_height']))
    # # # # # # PLOT CODE STARTS # # # # # #
    order = data[x].unique()
    p = sns.swarmplot(data=data,
                      x=x,
                      y=y,
                      hue=hue,
                      palette=palette,
                      order=order,
                      linewidth=0.5,
                      size=4)
    # # # # # # PLOT CODE ENDS # # # # # #

    # SCALING AND LIMITS STARTS >>>
    if x_scale != 'linear' or y_scale != 'linear':
        _scaler(p, x_scale, y_scale)

    if x_limit != None or y_limit != None:
        _limiter(data=data, x=x, y=y, x_limit=None, y_limit=y_limit)
    # <<< SCALING AND LIMITS ENDS

    # START OF TITLES >>>
    _titles(title, sub_title=sub_title)
    # <<< END OF TITLES

    _thousand_sep(p, ax, data, x, y)
    _footer(p, x_label, y_label, legend, n, save)
    # <<< FOOTER ENDS
    p.set_xticklabels(order, rotation=90)



def _thousand_sep(p, ax, data, x, y):

    '''Handles thousand separatos for tick labels.

    p | fig object | matplotlib figure object
    ax | axis object | a single axis object
    data | df | pandas dataframe with data
    x | str | column name for x
    y | str | column name for y

    NOTE: If x is list (e.g. hist or line) then x should be x[0].
          For example _thousand_sep(p, ax, data, x[0], y)

    '''

    from matplotlib import ticker

    # only apply if data is int
    if x != None:
        if isinstance(data[x].iloc[0], int):
            ax.get_xaxis().set_major_formatter(ticker.FuncFormatter(lambda x, p: format(int(x), ',')));

    # only apply if data is int
    if y != None:
        if isinstance(data[y].iloc[0], int):
            ax.get_yaxis().set_major_formatter(ticker.FuncFormatter(lambda x, p: format(int(x), ',')));



import pandas as pd
from IPython.core.display import display, HTML


def table(data,
          title="Descriptive Stats",
          sub_title="",
          table_width=630,
          indexcol_width=150,
          return_html=False):

    '''Displays a publication quality data table 
    with any number of columns and aspects. Works
    best for 6-7 or less columns datasets.

    data | DataFrame | a dataframe with three columns of numeric data
    title | str | the main title of the table
    sub_title | str | shown below the title
    table_width | int | the width of the table in pixels
    indexcol_width | int | the width of th index column in pixels.
    
    Width of the data columns is computed based on `table_width` and
    `indexcol_width` values. 

    '''
        
    html = '''
            <style type=\"text/css\">
            
            .tg {
                border-collapse:collapse;
                border-spacing:0;
                border:none;} 
            
            .tg td {
                font-family: Arial, sans-serif;
                font-size:14px;
                padding:10px 5px;
                border-style:solid;
                border-width:0px;
                overflow:hidden;
                word-break:normal;
                } 
                
            .tg th {
                font-family:Arial, sans-serif,
                sans-serif;
                font-size:14px;
                font-weight:normal;
                padding:10px 5px;
                border-style:solid;
                border-width:0px;
                overflow:hidden;
                word-break:normal;
                } 
                
            .tg .tg-index {
                font-family:Verdana, Geneva, sans-serif !important;
                text-align: left;
                padding-left: 10px;
                vertical-align:top
                } 
                
            .tg .tg-anay {
                    font-family:Verdana, Geneva, sans-serif !important;
                    text-align:right;
                    vertical-align:top
                    } 
                    
            .tg .tg-jua3 {
                    font-weight:bold;
                    font-family:Verdana, Geneva, sans-serif !important;
                    text-align:right;vertical-align:top
                    } 
            
            hr {
                height: 1px;
                background-color: #333;
                padding: 0px;
                margin: 0px;
            } 
            
            .hr2 {
                height: 2px !important;
                background-color: #333;
                }
                
            table {
                table-layout: fixed;
                width: 500px;
                border-style:hidden; 
                border-collapse: collapse;\"
                margin-top: 0px;
                padding-top: 0px;
            }
            
            .title {
                font-family: Arial, sans-serif;
                font-style: italic;
                font-size: 22px;
                font-weight: bold;
                padding-bottom: 0px;
                
                } 
                
            .sub_title {
                font-size: 18px;
                font-family: Arial, sans-serif;
                margin-top: 8px !important;
                padding-bottom: 12px;
                } 
                
            </style> 
            
            <table class=\"tg\"> 
            
            <colgroup> 
                <col style=\"width: _INDEXCOL_WIDTH_px\"> 
                <col style=\"width: _COL_WIDTH_px\"> 
                <col style=\"width: _COL_WIDTH_px\"> 
                <col style=\"width: _COL_WIDTH_px\"> 
                <col style=\"width: _COL_WIDTH_px\"> 
            </colgroup> 
            <p class='title'> _TABLE_TITLE_ </p> 
            <p class='sub_title'> _TABLE_SUBTITLE_ </p> 
            <hr align=\"left\", width=\"_TABLE_WIDTH_\"> 
        <tr>
            <th class=\"tg-index\"></th> 
        '''
    
    # add columns
    for col in data.columns:
        html += '<th class=\"tg-anay\">' + str(col) + '</th>' 

    for index in data.index:
        
        html += '</tr><tr><td class=\"tg-index\">' + str(index) + '</td>' 
        
        for col in data.columns:
            
            html += '<td class=\"tg-jua3\">' + f'{data.loc[index][col]:,}' + '</td>'
            
    html +='''    
        </tr> 
        </table> 
        <hr align=\"left\", width=\"_TABLE_WIDTH_\">
    
    '''
    
    col_width = int((table_width - indexcol_width) / len(data.columns))
    
    html = html.replace('_TABLE_TITLE_', title)
    html = html.replace('_TABLE_SUBTITLE_', sub_title)
    html = html.replace('_TABLE_WIDTH_', str(table_width))
    html = html.replace('_COL_WIDTH_', str(col_width))
    html = html.replace('_INDEXCOL_WIDTH_', str(indexcol_width))
    
    display(HTML(html))

    if return_html:
        return print(html)


import seaborn as sns
import pandas as pd

from ..style.template import _header
from ..utils.transform import _groupby


def compare(data,
            x,
            y,
            label_col,
            sort=False,
            transform_func=False,
            palette='default',
            style='astetik',
            dpi=72,
            title='',
            sub_title='',
            x_label='',
            y_label='',
            legend=True,
            x_scale='linear',
            y_scale='linear',
            x_limit=None,
            y_limit=None,
            save=False):

    '''COMPARE PLOT

    This needs to have a single row per entity, for example a patient
    or a state and so forth. It will be best if the columns are
    normalized data, percentages, or something similar.

   Inputs: at least 3
   Features: At least two continuous and one categorical.

    1. USE
    ======
    ast.compare(data=patients,
                x='hospital_stays',
                y=['died_hospital','died_out'],
                label_col='religion',
                transform=True)

    1. USE
    ======
    ast.box(data=patients,
            x='insurance',
            y='age',
            hue='expired')

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: a pandas dataframe

    x :: the main feature (continuous)

    y :: two or more features in a list (continuous)

    label_col :: the label column (categorical)

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
    sort :: True for ascending and False for descending

    transform_func :: If not false, the selected function such as
                      'mean' will be used to group by the label_col.
                      Available functions:
                        - 'median'
                        - 'mean'
                        - 'first'
                        - 'last',
                        - 'std',
                        - 'mode',
                        - 'max',
                        - 'min',
                        - 'sum',
                        - 'random'

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the astetik palettes:
                'default'
                'colorblind'
                'blue_to_red'
                'blue_to_green'
                'red_to_green'
                'green_to_red'
                'violet_to_blue'
                'brown_to_green'
                'green_to_marine'

                Or use any cmap, seaborn or matplotlib
                color or palette code, or hex value.

    style :: Use one of the three core styles:
                'astetik'     # white
                '538'         # grey
                'solarized'   # sepia

              Or alternatively use any matplotlib or seaborn
              style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'
    '''

    if transform_func != False:
        plot_data = _groupby(data, label_col, transform_func)
    else:
        plot_data = data

    if type(y) != type([]):
        y = [y]
    index_cols = y
    index_cols.insert(0, x)

    # HEADER STARTS >>>
    palette = _header(palette,
                      style,
                      n_colors=1,
                      dpi=dpi,
                      fig_height=None,
                      fig_width=None)
    # <<< HEADER ENDS

    # Make the PairGrid
    g = sns.PairGrid(plot_data.sort_values(x, ascending=sort),
                     x_vars=pd.Index(index_cols),
                     y_vars=[label_col],
                     size=10,
                     aspect=.25)

    # Draw a dot plot using the stripplot function
    g.map(sns.stripplot,
          size=10,
          orient="h",
          palette=palette,
          edgecolor="gray")

    g.set(xlabel="", ylabel="")
    titles = index_cols
    for ax, title in zip(g.axes.flat, titles):
        ax.set(title=title)
        ax.xaxis.grid(False)
        ax.yaxis.grid(True)

    sns.despine(left=True, bottom=True)



import seaborn as sns

from ..style.template import _header, _footer
from ..utils.utils import _scaler
from ..style.legend import _legend


def bargrid(data,
            x,
            y,
            hue=None,
            row=None,
            col=None,
            col_wrap=None,
            palette='default',
            style='astetik',
            dpi=72,
            title='',
            sub_title='',
            x_label='',
            y_label='',
            legend=True,
            legend_out=True,
            # legend_position=[],
            # legend_labels=None,
            x_scale='linear',
            y_scale='linear',
            x_limit=None,
            y_limit=None,
            save=False):

    '''BAR PLOT

    A multi-dimension bar plot that takes up to 5 features at a time.
    The most important thing to keep in mind is that only 'y' can and
    should be continuous. All other should be boolean/categoricalself.

    If you want to do a simple 1-d or 2-d barplot, you can do that with:

    - ast.bar() for 1-d
    - ast.bartwo() for 2-d

    This plot is only useful for the case where you have at least 4 dimensions
    you want to plot, and (again) all except 'y' are boolean/categorical.

    Inputs: 2 to 5
    Features: At least one continuous (or stepped) variable and rest
              can be categorical.

    1. USE
    ======
    ast.bars(data=patients,
              x='icu_days',
              y='insurance',
              hue='gender',
              col='religion',
              row='ethnicity')

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: pandas dataframe

    x :: x-axis data (categorical)

    y :: y-axis data (continuous or categorical)

    hue :: color highlight (categorical)

    row :: the comparison feature for side-by-side plots

    col :: the comparison feature for on top of each other plots

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
    None

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the astetik palettes:
                'default'
                'colorblind'
                'blue_to_red'
                'blue_to_green'
                'red_to_green'
                'green_to_red'
                'violet_to_blue'
                'brown_to_green'
                'green_to_marine'

                Or use any cmap, seaborn or matplotlib
                color or palette code, or hex value.

    style :: Use one of the three core styles:
                'astetik'     # white
                '538'         # grey
                'solarized'   # sepia

              Or alternatively use any matplotlib or seaborn
              style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'
    '''

    if hue != None:
        n_colors = len(data[hue].unique())
    else:
        n_colors = len(data[x].unique())

    # HEADER STARTS >>>
    palette = _header(palette,
                      style,
                      n_colors=n_colors,
                      dpi=dpi,
                      fig_height=None,
                      fig_width=None)
    # <<< HEADER ENDS
    p = sns.factorplot(data=data,
                       x=x,
                       y=y,
                       hue=hue,
                       row=row,
                       col=col,
                       col_wrap=col_wrap,
                       palette=palette,
                       legend=True,
                       legend_out=True,
                       size=4,
                       kind='bar')

    # SCALING AND LIMITS STARTS >>>
    if x_scale != 'linear' or y_scale != 'linear':
        _scaler(p, x_scale, y_scale)

    # FOOTER STARTS >>>
    #_legend(data[hue].unique(), legend, legend_labels, legend_position)
    _footer(p, x_label, y_label, save=save)



import pkg_resources
import numpy as np
import pandas as pd

import matplotlib as mpl
import matplotlib.pyplot as plt
from matplotlib.patches import Polygon
from matplotlib.collections import PatchCollection

from ..utils.exceptions import MissingImport

from ..style.formats import _thousand_sep
from ..utils.outliers import outliers

shapefile = pkg_resources.resource_filename(__name__, '../extras/countries')


def world(data,
          area_col,
          value_col,
          area_to_code=False,
          bin_mode='linear',
          log=None,
          title='',
          value_col_to_title=True,
          descripton='',
          num_colors=9,
          palette='Reds',
          filter_outlier=False):

    '''WORLD MAP PLOT

    Takes in data where one column is the values and another is either
    country or 3-alphabet country code according to ISO standard.

    USE
    ===

    world(data=emission,
          area_col='area',
          value_col=1999,
          area_to_code=True,
          title='Emission Intensity of Food Production (co2/kg)',
          value_col_to_title=False,
          palette='Reds',
          num_colors=9)


    area_col :: the column where is either country name or country code
    value_col :: the column with the values

    area_to_code :: must be True if area is not 3-alphabet code
    num_colors :: the number of colors to be used to describe intensity

    value_to_col_title :: Useful when the column is a year and you want to
                          have it shown in the title.

    log :: if true will use log values instead. Only works when bin_mode is
           linear.

    '''

    from geonamescache import GeonamesCache
    from ..utils.country_code import country_to_code

    data = data.copy(deep=True)

    if filter_outlier == True:
        data = outliers(data, value_col)

    if value_col_to_title == True:
        title = title + ' {}'.format(value_col)

    descripton.strip()

    if area_to_code == True:
        data[area_col] = data[area_col].apply(country_to_code)
        data.set_index(area_col, inplace=True)

    if data.index.name != area_col:
        data.set_index(area_col, inplace=True)

    # filter data based on geo codes
    gc = GeonamesCache()
    iso3_codes = list(gc.get_dataset_by_key(gc.get_countries(), 'iso3').keys())
    data = data.loc[iso3_codes]
    data = data[~data[value_col].isna()]
    data[value_col] = data[value_col].astype(int)

    # set plot stuff
    values = data[value_col].dropna()

    if log == True:
        values = np.log1p(values)
        data[value_col] = np.log(data[value_col])

    if bin_mode == 'linear':
        bins = np.linspace(values.min(), values.max(), num_colors)
    elif bin_mode == 'quantile':
        bins = np.nanpercentile(values, np.arange(0,
                                                  100,
                                                  num_colors))
    cm = plt.get_cmap(palette)
    scheme = [cm(i / num_colors) for i in range(num_colors)]

    # create the bin column
    data['temp'] = pd.cut(data[value_col], bins)
    cat_columns = data.select_dtypes(['category']).columns
    data['bin'] = data[cat_columns].apply(lambda x: x.cat.codes)
    data.drop('temp', axis=1, inplace=True)

    p = plt.figure(figsize=(17, 12))
    p.patch.set_facecolor('white')

    ax = p.add_subplot(111, frame_on=False)
    p.suptitle(title,
               color='grey',
               weight='bold',
               fontsize=26,
               y=.85)

    try:
        from mpl_toolkits.basemap import Basemap
        m = Basemap(lon_0=0, projection='robin')
    except ImportError:
        raise MissingImport("Install Basemap >> pip install git+https://github.com/matplotlib/basemap.git")

    m.drawmapboundary(color='w')

    m.readshapefile(shapefile, 'units', color='#444444', linewidth=.2)
    for info, shape in zip(m.units_info, m.units):
        iso3 = info['ADM0_A3']
        if iso3 not in data.index:
            color = '#dddddd'
        else:
            color = scheme[data.loc[iso3]['bin'].astype(int)]

        patches = [Polygon(np.array(shape), True)]
        pc = PatchCollection(patches)
        pc.set_facecolor(color)
        ax.add_collection(pc)

    # Cover up Antarctica so legend can be placed over it.
    ax.axhspan(0, 1000 * 1800, facecolor='w', edgecolor='w', zorder=2)

    # Draw color legend.
    ax_legend = p.add_axes([0.35, 0.24, 0.3, 0.03], zorder=3)
    cmap = mpl.colors.ListedColormap(scheme)
    cb = mpl.colorbar.ColorbarBase(ax_legend,
                                   cmap=cmap,
                                   ticks=bins,
                                   boundaries=bins,
                                   orientation='horizontal')

    cb.ax.set_xticklabels([str(round(i, 1)) for i in bins], rotation=45, ha='right')

    plt.annotate(descripton, xy=(-.8, -3.2), size=14, xycoords='axes fraction')

    _thousand_sep(p, ax, data, area_col, value_col)



def oned(data):

    '''1-d visualization for 1-d random sequences

    EXAMPLE
    =======

    test = random.sample(list(range(65000)),300)
    randhist(test)

    '''

    import matplotlib.pyplot as plt

    plt.figure(figsize=(12, 1))
    plt.eventplot(data,
                  orientation='horizontal',
                  colors='black',
                  linewidths=.8)
    plt.axis('off')
    plt.show()



import seaborn as sns

from ..style.template import _header, _footer
from ..style.formats import _thousand_sep
from ..style.titles import _titles
from ..utils.utils import factorplot_sizing


def count(data,
          x,
          sort=None,
          palette='default',
          style='astetik',
          dpi=72,
          title='',
          sub_title='',
          x_label='',
          y_label='',
          legend=True,
          x_scale='linear',
          y_scale='linear',
          x_limit=None,
          y_limit=None,
          save=False):

    '''COUNT PLOT

    A bar plot for counting values in a single feature. Useful for
    categoricals and stepped data.

    Inputs: 1
    Features: 1 categorical or stepped

    1. USE
    ======
    ast.count(x='insurance', data=patients, palette='Reds')

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: pandas dataframe

    x :: x-axis data (categorical or stepped)

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
    sort :: If True, will be sorted based on input values.

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the hand-crafted palettes:
                'default'
                'colorblind'
                'blue_to_red'
                'blue_to_green'
                'red_to_green'
                'green_to_red'
                'violet_to_blue'
                'brown_to_green'
                'green_to_marine'

                Or use any cmap, seaborn or matplotlib
                color or palette code, or hex value.

    style :: Use one of the three core styles:
                'astetik'     # white
                '538'         # grey
                'solarized'   # sepia

              Or alternatively use any matplotlib or seaborn
              style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'

    '''
    size, aspect = factorplot_sizing(data[x], auto=True)
    #aspect = int(len(data[x].unique()) / 5)

    if sort != None:
        sort = data[x].value_counts().index.values

    # HEADER STARTS >>>
    palette = _header(palette,
                      style,
                      n_colors=1,
                      dpi=dpi,
                      fig_height=None,
                      fig_width=None)

    p = sns.factorplot(data=data,
                       y=x,
                       palette=palette,
                       size=size,
                       aspect=aspect,
                       kind='count',
                       legend=legend,
                       legend_out=False,
                       order=sort)

    # FOOTER
    _titles(title, sub_title)
    _thousand_sep(p, p.ax, data, x, None)
    _footer(p, x_label, y_label, save=save)

    p.set_xticklabels(None, rotation=90)



import seaborn as sns

from ..style.template import _header, _footer


def multicount(data,
               x,
               hue=None,
               row=None,
               col=None,
               col_wrap=4,
               palette='default',
               style='astetik',
               dpi=72,
               title='',
               sub_title='',
               x_label='',
               y_label='',
               x_scale='linear',
               y_scale='linear',
               x_limit=None,
               y_limit=None,
               legend=True,
               save=False):

    '''MULTICOUNT PLOT

    A bar plot for counting values in a single feature and then
    comparing it with 'hue', 'col', and 'row' categorical variables.
    Prints out multiple plots. None of the values can be continuous.
    If you want to use continuous values then go for bargrid() instead.

    Inputs: 3 to 4
    Features: 1 continuous and 2 to 3 categoricals

    1. USE
    ======
    ast.multicount(data=patients,
                   x='expired',
                   hue='gender',
                   col='insurance')

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: pandas dataframe

    x :: x-axis data (categorical)

    y :: y-axis data (continuous or categorical)

    hue :: color highlight (categorical)

    col :: the side-by-side plot comparison feature

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
    col_wrap :: the number of plots to show per row

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the hand-crafted palettes:
                'default'
                'colorblind'
                'blue_to_red'
                'blue_to_green'
                'red_to_green'
                'green_to_red'
                'violet_to_blue'
                'brown_to_green'
                'green_to_marine'

                Or use any cmap, seaborn or matplotlib
                color or palette code, or hex value.

    style :: Use one of the three core styles:
                'astetik'     # white
                '538'         # grey
                'solarized'   # sepia

              Or alternatively use any matplotlib or seaborn
              style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'

    '''

    if row != None:
        col_wrap = None

    if hue != None:
        n_colors = len(data[hue].unique())
    else:
        n_colors = len(data[x].unique())

    # HEADER STARTS >>>
    palette = _header(palette,
                      style,
                      n_colors=n_colors,
                      dpi=dpi,
                      fig_height=None,
                      fig_width=None)
    # <<< HEADER ENDS

    p = sns.factorplot(data=data,
                       x=x,
                       y=None,
                       hue=hue,
                       row=row,
                       col=col,
                       col_wrap=col_wrap,
                       palette=palette,
                       size=4,
                       kind='count',
                       legend=legend,
                       legend_out=False)

    # FOOTER STARTS >>>
    _footer(p, x_label, y_label, save=save)



from IPython.core.display import display, HTML

from ..utils.utils import _sort_strings


def text(data,
         column,
         sort=False,
         title='_column_name_',
         max_rows=20,
         contains=''):

    '''PRETTY TEXT PRINTER

    USE
    ---
    text(data=df, column='Name', contains='Miss')

    This will yield a printed out list of string
    items in the Name column where 'Miss' is present.

    PARAMETERS
    ----------
    data ::  a dataframe with the data
    column :: a column of values to print
    title :: the title to be printed before content
    max_rows :: max number of rows to be printed
    sort :: sorts 'asc' or 'desc' when not 'False'
    contains :: string to be matched (accepts regex)
    '''

    if title == '_column_name_':
        title = column

    if contains != '':
        data = data[data[column].str.contains(contains) == True]

    if sort != False:
        data = _sort_strings(data, column, sort)

    if max_rows >= len(data):
        max_rows = len(data)

    # reset the index to avoid issues
    data = data.copy(deep=True)
    data.reset_index(inplace=True)

    # display the content
    display(HTML("<h4>" + str(title) + "</h4>"))
    for row in range(max_rows):
        s = str(data[column][row])
        html = display(HTML("<font size=2 face=Verdana>" + s + "</font>"))

    return html



import numpy as np
import seaborn as sns
from matplotlib.colorbar import cm


def color_picker(palette, center='light', n_colors=10, show=False):

    '''COLOR PICKER
    Provides access to several color palettes that will
    serve the purpose of most usecases with astetik plots.

    PARAMETERS
    ----------
    palette :: name of the palette (see list below)
    center :: if the center of the palette is 'light' or 'dark'
    n :: number of colors in the color
    show :: if the palette will be shown for visual reference

    PALETTES
    --------
    NOTE: all two color palettes will reflect the first color
    in the selected range. For example blue_to_red will have
    two shades of blue.

    'blue_to_red'
    'blue_to_green'
    'red_to_green'
    'green_to_red'
    'violet_to_blue'
    'brown_to_green'
    'green_to_marine'
    '''

    if palette == 'default':
        palette = 'blue_to_red'

    # handle the case where few colors only are used
    if n_colors <= 5:
        n_input = n_colors
        n_colors = 8  # this decides how dark colors will be
    else:
        n_input = n_colors

    if palette == 'blue_to_red':
        out = sns.color_palette("RdBu_r", n_colors=n_colors)
    elif palette == 'blue_to_green':
        out = sns.color_palette("GnBu_d", n_colors=n_colors)
    elif palette == 'red_to_green':
        out = sns.diverging_palette(16, 180, sep=5, center=center, n=n_colors)
    elif palette == 'green_to_red':
        out = sns.diverging_palette(180, 16, sep=5, center=center, n=n_colors)
    elif palette == 'violet_to_blue':
        out = sns.diverging_palette(1, 255, sep=5, center=center, n=n_colors)
    elif palette == 'brown_to_green':
        out = sns.diverging_palette(50, 100, sep=5, center=center, n=n_colors)
    elif palette == 'green_to_marine':
        out = sns.diverging_palette(100, 200, sep=5, center=center, n=n_colors)

    if n_input == 1:
        out = out[0]

    elif n_input == 2:
        out = out[:2]

    if show == True:
        sns.palplot(out)

    if np.ndim(out) == 1:
        out = [out]

    return out


def color_blind(mode='colorblind'):

    '''COLOR BLIND COLORS

    Provides a color palette that is colorblind friendly.
    '''

    if mode == 'colorblind':

        colors = [[0, 0, 0],
                  [255/255, 255/255, 109/255],
                  [255/255, 109/255, 182/255],
                  [0, 73/255, 73/255],
                  [0, 146/255, 146/255],
                  [255/255, 182/255, 119/255],
                  [73/255, 0, 146/255],
                  [0, 109/255, 219/255],
                  [182/255, 109/255, 255/255],
                  [109/255, 182/255, 255/255],
                  [182/255, 219/255, 255/255],
                  [146/255, 0, 0],
                  [146/255, 73/255, 0],
                  [219/255, 209/255, 0],
                  [36/255, 255/255, 36/255]]

    elif mode == 'colorblind6':
        colors = [[0, 0, 0],
                  [230/255, 159/255, 0],
                  [86/255, 180/255, 233/255],
                  [0, 158/255, 115/255],
                  [213/255, 94/255, 0],
                  [0, 114/255, 178/255]]

    elif mode == 'colorblind1':
        colors = [[222/255, 188/255, 146/255],
                  [251/255, 175/255, 148/255],
                  [131/255, 215/255, 142/255],
                  [225/255, 191/255, 147/255],
                  [250/255, 109/255, 81/255],
                  [101/255, 170/255, 53/255],
                  [204/255, 146/255, 68/255],
                  [221/255, 51/255, 48/255],
                  [95/255, 130/255, 24/255],
                  [149/255, 115/255, 32/255],
                  [164/255, 23/255, 30/255],
                  [61/255, 105/255, 22/255],
                  [119/255, 81/255, 24/255]]


    return colors


def cmaps(cmap):

    if cmap == 'paired':
        cmap = cm.Paired
    elif cmap == 'jet':
        cmap = cm.jet
    elif cmap == 'prism':
        cmap = cm.prism
    elif cmap == 'RdYlGn':
        cmap = cm.RdYlGn
    elif cmap == 'seismic':
        cmap = cm.seismic
    elif cmap == 'coolwarm':
        cmap = cm.coolwarm
    elif cmap == 'inferno':
        cmap = cm.inferno
    elif cmap == 'plasma':
        cmap = cm.plasma
    elif cmap == 'OrRd':
        cmap = cm.OrRd
    elif cmap == 'tab20c':
        cmap = cm.tab20c

    return cmap


def _label_to_hex(label, n_colors):

    hex = sns.color_palette(label, n_colors)
    return hex.as_hex()



import glob
import os
import numpy as np
# EXCEPTIONAL IMPORT #

import matplotlib.pyplot as plt

from IPython.display import clear_output


class Animation():

    '''ANIMATED CHARTS

    Supports dataformat where x and y are compared against each other, and
    label_col provides the title (e.g. year) for each frame in the animation.

    y and x are used for the legend labels when relevant.

    '''

    def __init__(self, data, x, y, label_col, plot_type, filename='animation',
                 dpi=72, palette='default'):

        self.plot_type = plot_type
        self.data = data
        self.x = x
        self.y = y
        self.label_col = label_col
        self.palette = palette
        self.filename = filename
        self.frames = len(data)
        self.dpi = dpi

        # RUNTIME
        self._ = self._create_plots()
        self._ = self._create_gif()

    def _create_plots(self):

        for i in range(self.frames):
            data = np.array([self.data[self.x][i], self.data[self.y][i]]).astype(int)
            self.plot_type(data,
                           labels=[self.x, self.y],
                           palette=self.palette,
                           sub_title=self.data[self.label_col][i],
                           save=True,
                           dpi=self.dpi)
            clear_output()
            plt.show()

    def _create_gif(self):

        gif_name = self.filename
        file_list = glob.glob('astetik_*.png')
        list.sort(file_list, key=lambda x: int(x.split('_')[1].split('.png')[0]))

        with open('image_list.txt', 'w') as file:
            for item in file_list:
                file.write("%s\n" % item)

        os.system('convert @image_list.txt {}.gif'.format(gif_name)) # magick on windows



import warnings

import seaborn as sns
import matplotlib.pyplot as plt

from ..style.style import params
from ..style.titles import _titles
from ..style.formats import _thousand_sep

from ..style.template import _header, _footer
from ..utils.utils import _limiter, _scaler


def hist(data,
         x,
         bins=True,
         dropna=False,
         vertical=False,
         kde=False,
         norm_hist=False,
         alpha=1,
         palette='default',
         style='astetik',
         dpi=72,
         title='',
         sub_title='',
         x_label='',
         y_label='',
         legend=False,
         x_scale='linear',
         y_scale='linear',
         x_limit=None,
         y_limit=None,
         save=False):

    '''HISTOGRAM

    Create a classic histogram. Note that nans will be
    dropped automatically as otherwise seaborn will throw
    an error.

    Inputs: 1
    Features: One continuous

    1. USE
    ======
    ast.hist(data=df,
             x='Fare',
             style='astetik)

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: pandas dataframe

    x :: x-axis data (continuous)

    y :: x-axis overlap data (continuous or categorical)

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
     bins= The number of bins to be shown. If True, will be automatic.

     dropna= True if na values should be dropped first.

     vertical= The orientation of the plot 'v' or 'h'

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the hand-crafted palettes:
                'default'
                'colorblind'
                'blue_to_red'
                'blue_to_green'
                'red_to_green'
                'green_to_red'
                'violet_to_blue'
                'brown_to_green'
                'green_to_marine'

                Or use any cmap, seaborn or matplotlib
                color or palette code, or hex value.

    style :: Use one of the three core styles:
                'astetik'     # white
                '538'         # grey
                'solarized'   # sepia

              Or alternatively use any matplotlib or seaborn
              style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'

    '''
    warnings.simplefilter("ignore")


    if bins == True:
        if isinstance(x, list) is False:
            bins = int(len(data[x].unique()) / 10) + 5
        else:
            bins = 10

    if dropna is True:
        data = data[data[x].isna() == False]

    # PLOT
    p, ax = plt.subplots(figsize=(params()['fig_width'],
                                  params()['fig_height']))

    if isinstance(x, list) is False:
        x = [x]

    n_colors = len(x)
    palette = _header(palette, style, n_colors=n_colors, dpi=dpi)

    for i in range(len(x)):
        p = sns.distplot(data[x[i]].dropna(),
                         bins=bins,
                         norm_hist=norm_hist,
                         kde=kde,
                         color=palette[i],
                         vertical=vertical,
                         hist_kws=dict(alpha=alpha))

    # SCALING AND LIMITS
    if x_scale != 'linear' or y_scale != 'linear':
        _scaler(p, x_scale, y_scale)

    if x_limit != None or y_limit != None:
        _limiter(data=data, x=x, y=x, x_limit=x_limit, y_limit=y_limit)

    # HEADER
    _thousand_sep(p, ax, data, x[0], None)
    _titles(title, sub_title)
    _footer(p, x_label, y_label, save=save)



def _legend(x, legend, legend_labels, legend_position):

    import matplotlib.pyplot as plt

    if legend:
        if legend_labels != None:
            x = legend_labels

        if len(legend_position) == 0:
            plt.legend(x, loc=1, ncol=1, bbox_to_anchor=(1.25, 1.0))
        else:
            plt.legend(x, loc=legend_position[0], ncol=legend_position[1])



import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from matplotlib import ticker

from ..style.formats import _thousand_sep
from ..style.style import params
from ..style.titles import _titles
from ..style.template import _header, _footer
from ..style.legend import _legend
from ..utils.utils import _limiter, _scaler
from ..utils.utils import multicol_transform
from ..utils.datetime import date_handler


def line(data,
         x=None,
         y=None,
         xtick_labels=None,
         highlight_x=None,
         interval=False,
         interval_func=None,
         time_frame=None,
         dropna=False,
         median_line=False,
         drawstyle='default',
         linestyle=None,
         linewidth=None,
         markerstyle=None,
         smooth=None,
         legend_labels=None,
         annotate_line_end=False,
         annotate_text=None,
         annotate_xy=(),
         annotate_text_xy=(),
         palette='default',
         alpha=1,
         style='astetik',
         dpi=72,
         title='',
         sub_title='',
         titles_align='center',
         x_label='',
         y_label='',
         legend=False,
         legend_position=[],
         x_scale=None,
         y_scale=None,
         x_limit=None,
         y_limit=None,
         save=False):

    '''TIMESERIES LINE PLOT

    A line plot for one or more columns all with a comparable
    value, in a time sequence. IF 'x' is None, all columns except
    'y' will be included.

    1.USE
    =====
    line_plot(data=ldata,
              x='value',
              linestyle='dashdot',
              palette='colorblind',
              title="The main title comes here",
              sub_title="Suptibtle comes here")

    Inputs: 1 or more continuous and an optional timestamp
    Features: Continuous and optional datetime format

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: pandas dataframe

    x :: one or more columns of data

    y :: a single timeseries column (no need to be dt)
         and if y is not defined, then a sequence will
         be automatically generated as time labels.

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
    interval :: If not False, should be number of minutes per
                 sample as int or one of the presets:
                 - 'quarter'
                 - 'half',
                 - 'full' (days)
                 - 'week'
                 - 'month' (30 days)
                 - 'year'.

    interval_func :: The grouping by function that will be used:

                        'median', 'mean', 'mode',
                        'first', 'last', 'std', 'mode'
                        'max', 'min', 'sum', 'random',
                        or 'freq'

    time_frame :: the time frame to be used for x-axis labels:

                  'year', 'month', 'day', 'hour', 'minute', 'second'

    median_line :: If True, a median line will be drawn

    drawstyle :: 'default', 'steps', 'steps-pre','steps-mid' or 'steps-post'

    linestyle :: 'solid', 'dashed', 'dashdot' , 'dotted'

    markerstyle :: ".", ",", "o", "+", "x", "|", "_", "^", "v"

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the hand-crafted palettes:
               'default'
               'colorblind'
               'blue_to_red'
               'blue_to_green'
               'red_to_green'
               'green_to_red'
               'violet_to_blue'
               'brown_to_green'
               'green_to_marine'

               Or use any cmap, seaborn or matplotlib
               color or palette code, or hex value.

    alpha :: Color saturation (float)

    style :: Use one of the three core styles:
               'astetik'     # white
               '538'         # grey
               'solarized'   # sepia

             Or alternatively use any matplotlib or seaborn
             style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    titles_align :: by default 'center'

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'

    legend_position | list | optionally pass legend `loc` and `ncol` values.

    '''

    data = data.copy(deep=True)

    # START OF PLOT SPECIFIC >>>

    if isinstance(x, list) is False:
        x = [x]

    lines = len(x)

    if dropna:
        data = data[data[x].isna() == False]

    if y == None:
        data[y] = range(len(data))

    if interval != False:
        data = multicol_transform(transform='interval',
                                  data=data,
                                  x=x,
                                  y=y,
                                  func=interval_func,
                                  freq=interval)
    if smooth is not None:

        from scipy.ndimage import gaussian_filter1d
        data[x] = data[x].apply(gaussian_filter1d, sigma=smooth)

    if markerstyle is None:
        markers = ["o", "+", "x", "|", "1", "8", "s", "p",
                   "o", "+", "x", "|", "1", "8", "s", "p"]
    else:
        markers = []
        for i in range(lines):
            markers.append(markerstyle)
    # <<< END OF PLOT SPECIFIC

    if linestyle is None:
        linestyle = ['solid'] * lines
    elif isinstance(linestyle, str):
        linestyle = [linestyle] * lines

    if linewidth is None:
        linewidth = [2] * lines
    elif isinstance(linewidth, list) is False:
        linewidth = [linewidth] * lines

    if highlight_x is not None:
        linestyle = ['--'] * lines
        linestyle[x.index(highlight_x)] = 'solid'
        linewidth = [2] * lines
        linewidth[x.index(highlight_x)] = 4
    
    # START OF HEADER >>>
    palette = _header(palette, style, n_colors=lines, dpi=dpi)
    # <<< END OF HEADER

    p, ax = plt.subplots(figsize=(params()['fig_width'] + 2,
                                  params()['fig_height']))

    # # # # PLOT STARTS # # # #
    for i in range(lines):
        p = plt.plot(data[y],
                     data[x[i]],
                     marker=markers[i],
                     drawstyle=drawstyle,
                     linestyle=linestyle[i],
                     c=palette[i],
                     linewidth=linewidth[i],
                     markersize=7,
                     markeredgewidth=1,
                     mfc='white',
                     rasterized=True,
                     aa=True,
                     alpha=alpha)

    if len(annotate_xy) > 0:

        ax.annotate(annotate_text,
                    xy=(annotate_xy[0], annotate_xy[1]),
                    xycoords='data',
                    xytext=(annotate_text_xy[0], annotate_text_xy[1]),
                    textcoords='axes fraction',
                    color='#888888',
                    size=15,
                    arrowprops=dict(facecolor='#888888',
                                    shrink=0.05,
                                    color='#888888',
                                    lw=2),
                    horizontalalignment='right', 
                    verticalalignment='top')

    if annotate_line_end:

        for i, col in enumerate(x):

            ax.annotate(col + ' ' + str(round(data[col][-1:].values[0], 2)),
                        xy=(len(data[col]), data[col][-1:].values), 
                        xytext=(6, data[col][-1:].values), 
                        color=palette[i], 
                        xycoords='data', 
                        textcoords="offset points",
                        size=14,
                        va="center")

    plt.gcf().axes[0].yaxis.get_major_formatter().set_scientific(False)

    # SCALING
    if y_scale != None or x_scale != None:
        for i in range(lines):
            _scaler(p[i], x_scale, y_scale)

    # # # # PLOT ENDS # # # #
    if median_line:
        if len(x) > 1:
            print("You can only have mean line with single line")
        else:
            x_median = data[x].median()
            x_median = np.full(len(data), x_median)
            plt.plot(data[y], x_median)

    # DATETIME FORMAT
    if time_frame != None:
        data[y] = pd.to_datetime(data[y])
        date_handler(data[y], ax, time_frame)

    # LIMITS
    if x_limit != None or y_limit != None:
        _limiter(data=data, x=x, y='_R_E_S_', x_limit=None, y_limit=y_limit)

    _thousand_sep(p, ax, data, y, x[0])
    _titles(title, sub_title=sub_title, location=titles_align)
    _footer(p, x_label, y_label, save=save, tight=False)
    _legend(x, legend, legend_labels, legend_position)

    ax.xaxis.set_major_locator(ticker.MaxNLocator(nbins=5, integer=True))

    if xtick_labels is not None:
        _len_ = len(xtick_labels)
        _picks_ = list(range(0, _len_, int(_len_ / 7)))
        plt.xticks(ticks=_picks_, labels=xtick_labels[_picks_])
